#property library

#include <MyLib\Arrays\DblApproxArray.mqh>
#include <MyLib\Common\IniCore.mq5>
#include <MyLib\DataStructures\StringMatrix.mq5>

static const string CDblApproxArray::APPROX_POWER_ENTRY = "ApproxPower";

CDblApproxArray::CDblApproxArray(uint uiAddBy,ELSMType ltApproxType):m_xpaArray(uiAddBy)
{
   SetMyObjectType(MOT_DBLAPPROX_ARRAY);
   m_ltApproxType = ltApproxType;
};

double CDblApproxArray::At(uint uiIdx,bool & bElementWasFound) const
{
   uint uiFoundIdx = UINT_MAX;
   
   double dRes = m_xpaArray.Approximate(uiIdx,m_ltApproxType,uiFoundIdx);
   
   if(uiFoundIdx == UINT_MAX)
      bElementWasFound = false;
   else
      bElementWasFound = true;
   
   return(dRes);      
};


double CDblApproxArray::AtWithStore(uint uiIdx,bool & bElementWasFound)
{
   uint uiFoundIdx = UINT_MAX;
   
   double dRes = m_xpaArray.Approximate(uiIdx,m_ltApproxType,uiFoundIdx);
   
   if(uiFoundIdx == UINT_MAX)
      {
      bElementWasFound = false;
      
      bool bRes = Update(uiIdx,dRes);
      
      ASSERT(bRes);
      }
   else
      bElementWasFound = true;
   
   return(dRes);      
};

bool CDblApproxArray::Update(uint uiIdx,double dElement)
{
   uint uiPlace = UINT_MAX;

   if(m_xpaArray.FindPointByX(uiIdx,uiPlace) == false)
      return(m_xpaArray.AddPointByXY(uiIdx,dElement));

   SXYPoint xpElement;
   
   xpElement.m_dX = uiIdx;
   xpElement.m_dY = dElement;
      
   m_xpaArray.UpdateElement(uiPlace,xpElement);
   
   return(true);      
};

bool CDblApproxArray::DeleteElement(uint uiIdx)
{
   uint uiPlace = UINT_MAX;

   if(m_xpaArray.FindPointByX(uiIdx,uiPlace) == false)
      return(false);
   
   m_xpaArray.DeleteElement(uiIdx);
   
   return(true);
};

bool CDblApproxArray::CopyAllFrom(CArrayDouble* padArray)
{
   ASSERT_STDPOINTER(padArray);
   
   DeleteContents();
   
   for(int iI=0; iI<padArray.Total(); ++iI)
      { RETURN_IF_FALSE(m_xpaArray.AddPointByXY(iI,padArray.At(iI))); };
   
   return(true);      
};

bool CDblApproxArray::CopyNecessaryOnlyFrom(CArrayDouble* padArray,double dMaxPriceDiff,double dMaxRelDiff)
{
   ASSERT_STDPOINTER(padArray);
   
   DeleteContents();
   
   if(padArray.Total()<=NULL)
      return(true);
   
   CXYPointsArray xpaBuffer;
   
   for(int iI=0; iI<padArray.Total(); ++iI)
      { RETURN_IF_FALSE(xpaBuffer.AddPointByXY(iI,padArray.At(iI))); };
   
   return(m_xpaArray.GetNecessaryOnlyFrom(GetPointer(xpaBuffer),m_ltApproxType,dMaxPriceDiff,dMaxRelDiff));
};

bool CDblApproxArray::CopyTo(CArrayDouble* padArray,bool bApprox,uint uiMaxToCopy) const
{
   ASSERT_STDPOINTER(padArray);
   
   padArray.Clear();
   
   if(GetRealPoints().Total() == NULL || uiMaxToCopy == NULL)
      return(true);
   
   uint uiMaxI = uiMaxToCopy-1;
   if(uiMaxI > GetMaxIdx())
      uiMaxI = GetMaxIdx();
   
   uint uiFoundPointIdx = UINT_MAX;
   double dValue = EMPTY_VALUE;
   
   for(uint uiI=0; uiI<=uiMaxI; ++uiI)
      {
      dValue = m_xpaArray.Approximate(uiI,m_ltApproxType,uiFoundPointIdx);
      
      if(bApprox == false && uiFoundPointIdx == UINT_MAX)
         dValue = EMPTY_VALUE;
         
      
      RETURN_IF_FALSE(padArray.Add(dValue));
      };
      
   return(true);      
};


bool CDblApproxArray::Store(CINIFileI* pifINIFile,bool bDeleteContents) const
{
   ASSERT(pifINIFile!=NULL);
   
   if(bDeleteContents)
      pifINIFile.DeleteContents();
      
   RETURN_IF_FALSE(CINICore::CheckRes(pifINIFile.WriteLong(CXYPointsArray::COMMON_SECTION,APPROX_POWER_ENTRY,m_ltApproxType))); 
         
   return(m_xpaArray.Store(pifINIFile));   
};

bool CDblApproxArray::Load(CINIFileI* pifINIFile)
{
   ASSERT(pifINIFile!=NULL);
   
   DeleteContents();

   long lPower = WRONG_VALUE;

   RETURN_IF_FALSE(pifINIFile.ReadLong(CXYPointsArray::COMMON_SECTION,APPROX_POWER_ENTRY,lPower));
   
   if(lPower != LSM_FLAT && lPower != LSM_TREND && lPower != LSM_PARABOLIC && lPower != LSM_CUBIC)
      return(false);
      
   m_ltApproxType = (ELSMType)lPower;

   return(m_xpaArray.Load(pifINIFile));
};

bool CDblApproxArray::ExportContentsToCSVMatrix(CStringMatrix* psmStrMatrix,bool bWithApprox,bool bNeedHeader) const
{
   return(m_xpaArray.ExportCSVTo(psmStrMatrix,bNeedHeader,m_ltApproxType));
};


