#property library

#include <MyLib\DebugOrRelease\DebugSupport.mqh>
#include <MyLib\Arrays\MyArray.mqh>

bool CMyArrayDouble::SetNewSize(int iNewSize)
{
   ASSERT(iNewSize >=0);

   if(iNewSize<0) return(false);

   int iNeedSize = m_step_resize*(1+iNewSize/m_step_resize);
   int iRes;
   
   if(m_data_max < iNeedSize)
      {
      iRes = ArrayResize(m_data,iNeedSize);
      
      if(iRes < 0)
         return(false);
         
      m_data_max = iRes;
      }
   
   m_data_total=iNewSize;

   return(m_data_max==iNeedSize);
};

void CMyArrayDouble::TraceArray(CArrayDouble* padArray,string strStartMessage = NULL,string strStopMessage = NULL)
{
   if(strStartMessage != NULL)
      TRACE(strStartMessage);
      
   for(int iI=0; iI<padArray.Total(); ++iI)
      TRACE_DOUBLE_FULL("Element " + IntegerToString(iI) + ": ",padArray.At(iI));     

   if(strStopMessage != NULL)
      TRACE(strStopMessage);
   
};

int CMyArrayDouble::CopyTo(double& dArray[])
{
   if(Total()==0)
      {
      ArrayFree(dArray);
      return(NULL);
      };
   
   int iInSize = ArraySize(m_data);
   
   if(iInSize < 0 || iInSize < Total())
      return(WRONG_VALUE);

   if(ArrayResize(dArray,Total())!=Total())
      return(WRONG_VALUE);
   
   if(ArrayCopy(dArray,m_data,0,0,Total())!=Total())
      return(WRONG_VALUE);

   return(Total());      
};

double CMyArrayDouble::GetMaxDiff(CArrayDouble* padFirst,CArrayDouble* padSecond,bool bCheckEmpty,uint & uiMaxIdx)
{
   ASSERT_STDPOINTER(padFirst);
   ASSERT_STDPOINTER(padSecond);

   double dMaxDiff = NULL;
   uiMaxIdx = UINT_MAX;
   
   int iMaxTest = MathMax(padFirst.Total(),padSecond.Total());
   double dFirst = EMPTY_VALUE;
   double dSecond = EMPTY_VALUE;
   double dCurDiff = EMPTY_VALUE;
   
   for(int iI=0; iI<iMaxTest; ++iI)
      {
      if(iI<padFirst.Total())
         dFirst = padFirst.At(iI);
      
      if(iI<padSecond.Total())
         dSecond = padSecond.At(iI);
         
      if(bCheckEmpty)
         {
         if(dFirst == EMPTY_VALUE)
            {
            if(dSecond == EMPTY_VALUE)
               continue;
            else
               {
               if(MathAbs(dSecond) > dMaxDiff)
                  {
                  dMaxDiff = MathAbs(dSecond);
                  uiMaxIdx = iI;
                  };
                  
               continue;   
               };   
            };             

         if(dSecond == EMPTY_VALUE)
            {
            if(MathAbs(dFirst) > dMaxDiff)
               {
               dMaxDiff = MathAbs(dFirst);
               uiMaxIdx = iI;
               };
            continue;   
            };
         };         
      
      dCurDiff = MathAbs(dFirst-dSecond);
      
      if(dCurDiff > dMaxDiff)
         {
         dMaxDiff = dCurDiff;
         uiMaxIdx = iI;
         };
      }; // for(int iI=0; iI<iMaxTest; ++iI)
   
   return(dMaxDiff);   
};


bool CMyArrayInt::SetNewSize(int iNewSize)
{
   ASSERT(iNewSize >=0);

   if(iNewSize<0) return(false);

   int iNeedSize = m_step_resize*(1+iNewSize/m_step_resize);
   int iRes;
   
   if(m_data_max < iNeedSize)
      {
      iRes = ArrayResize(m_data,iNeedSize);
      
      if(iRes < 0)
         return(false);
         
      m_data_max = iRes;
      }
   
   m_data_total=iNewSize;

   return(m_data_max==iNeedSize);
};

void CMyArrayInt::TraceArray(CArrayInt* paiArray,string strStartMessage = NULL,string strStopMessage = NULL)
{
   if(strStartMessage != NULL)
      TRACE(strStartMessage);
      
   for(int iI=0; iI<paiArray.Total(); ++iI)
      TRACE_INTEGER("Element " + IntegerToString(iI) + ": ",paiArray.At(iI));     

   if(strStopMessage != NULL)
      TRACE(strStopMessage);
   
};

CMyArrayString::CMyArrayString()
{
   Sort();
};


bool CMyArrayString::InsertSort(string  strElement,uint & rPlace)
{
   int pos;
//--- check
   if(!IsSorted())
      return(false);
      
//--- check/reserve elements of array
   if(!Reserve(1))
      return(false);
      
//--- if the array is empty, add an element
   if(m_data_total==0)
      {
      m_data[m_data_total++]=strElement;
      rPlace = 0;
      return(true);
      }
      
//--- find position and insert
   pos=QuickSearch(strElement);
   
   if(m_data[pos]>strElement)
      {
      Insert(strElement,pos);
      rPlace = pos;
      }
   else
      {
      Insert(strElement,pos+1);
      rPlace = pos+1;
      }
      
//--- restore the sorting flag after Insert(...)
   m_sort_mode=0;
   
   
//--- successful
   return(true);
   
};

