#property library

#include <MyLib\Common\GetLastError.mqh>
#include <MyLib\Arrays\PointsArray.mqh>
#include <MyLib\Math\AvrCounter.mq5>

bool CPointsArray::AddElement(SWPriceIdxPoint& dpiPoint)
{
   RETURN_IF_FALSE(SetSize(GetSize()+1));
   
   SetElement(GetSize()-1,dpiPoint);
   
   return(true);
};  

void CPointsArray::SetEqualWeights(double dWeight)
{
   for(uint uiI=0; uiI<GetSize(); ++uiI)
      m_dipPoints[uiI].m_dPointWeight = dWeight;
};

void CPointsArray::SetEqualGrowWeights(double dGrowFactor,double dZeroIdxWeight)
{
   double dCurWeight = dZeroIdxWeight;
   
   for(uint uiI=0; uiI<GetSize(); ++uiI)
      {
      m_dipPoints[uiI].m_dPointWeight = dCurWeight;
      dCurWeight += dGrowFactor;
      };
};

void CPointsArray::SetEqualMultGrowWeights(double dMultGrowFactor,double dZeroIdxWeight)
{
   double dCurWeight = dZeroIdxWeight;
   
   for(uint uiI=0; uiI<GetSize(); ++uiI)
      {
      m_dipPoints[uiI].m_dPointWeight = dCurWeight;
      dCurWeight *= dMultGrowFactor;
      };
};

void CPointsArray::SetMyEMAWeights(double dMinWeight,double dMinToMaxDistribution)
{
   double dMinCorr = 1;
   double dMaxCorr = 1;

   if(dMinToMaxDistribution != EMPTY_VALUE)
      {
      if(dMinToMaxDistribution <= 0)
         dMaxCorr = 0;
      else
         if(dMinToMaxDistribution >= 1)
            dMinCorr = 0;
         else
            {
            dMinCorr = 1 - dMinToMaxDistribution;
            dMaxCorr = dMinToMaxDistribution;
            };   
      };

   ASSERT(dMinCorr >= 0 && dMinCorr<=1);
   ASSERT(dMaxCorr >= 0 && dMaxCorr<=1);

   if(GetSize() == 0)
      return;

   double dMyAlpha = CAverageCounterCore::CountMyEMA_Alpha(GetSize());

   ASSERT(dMyAlpha < 1);

   double dCurWeight = dMinWeight;
   
   for(uint uiI=GetSize(); uiI>0; --uiI)
      {
      ASSERT(uiI <= GetSize());
      
      if(uiI ==  GetSize())
         m_dipPoints[uiI-1].m_dPointWeight = dCurWeight*dMinCorr;
      else
         if(uiI == 1)
            m_dipPoints[uiI-1].m_dPointWeight = dCurWeight*dMaxCorr;
         else   
            m_dipPoints[uiI-1].m_dPointWeight = dCurWeight;
         
      dCurWeight /= dMyAlpha;
      };
      
};




bool CPointsArray::FindLinearPointByBarIdx(int iBarIdx,uint &ruiArrayIdx)
{
   uint uiSize = GetSize();

   for(uint uiI=0; uiI<uiSize; ++uiI)
      {
      if(At(uiI).m_iBarIdx == iBarIdx)
         {
         ruiArrayIdx = uiI;
         return(true);
         }
      };

   ruiArrayIdx = UINT_MAX;
   return(false);
};
