#property library

#include <MyLib\DebugOrRelease\DebugSupport.mqh>
#include <MyLib\Arrays\QueueT.mqh>

template<typename T>
CQueueT::CQueueT()
{
   SetMyObjectType(MOT_QUEUE_T);
   
   DeleteContents(false);
};

template<typename T>
void CQueueT::DeleteContents(bool bClearArray)
{
   m_uiMaxQueueSize = 0;
   m_uiDataSize = 0;
   m_iTailPointer = INT_MAX;
   
   if(bClearArray)
      ArrayFree(m_atObjects);
}


template<typename T>
bool CQueueT::Init(uint uiMaxQueueSize)
{
   ASSERT(uiMaxQueueSize > 0);
   
   int iResSize = ArrayResize(m_atObjects,uiMaxQueueSize);
   
   if(iResSize != uiMaxQueueSize)
      return(false);
      
   m_uiMaxQueueSize =  uiMaxQueueSize;
   
   RemoveElements();
   
   return(true);  
};

template<typename T>
bool CQueueT::AddToTail(T & tObject)
{
   ASSERT(m_uiMaxQueueSize > 0);
   ASSERT(ArraySize(m_atObjects) == m_uiMaxQueueSize);

   if(m_uiDataSize == 0)
      {
      m_iTailPointer = 0;
      m_atObjects[0] = tObject; 
      m_uiDataSize = 1;
      return(true);
      };

   ASSERT(m_uiDataSize <= m_uiMaxQueueSize);
   ASSERT(m_iTailPointer < (int)m_uiMaxQueueSize);

   bool bRes = false;

   ++m_iTailPointer;
   
   if(m_iTailPointer == (int)m_uiMaxQueueSize)
      m_iTailPointer = 0;
      
   if(m_uiDataSize<m_uiMaxQueueSize)
      {
      ++m_uiDataSize;
      bRes = true;
      };

   m_atObjects[m_iTailPointer] = tObject; 

   return(bRes);
};
template<typename T>
bool CQueueT::ReplaceTail(T & tObject,bool bAddIfEmpty)
{
   if(m_uiDataSize > 0)
      {
      m_atObjects[m_iTailPointer] = tObject; 
      return(true);
      };
   
   if(bAddIfEmpty == false)
      return(false);

   AddToTail(tObject);

   return(true);
};



template<typename T>
T CQueueT::GetObjectFromHead(uint uiIdxFromHead) const
{
   ASSERT(m_uiDataSize > 0);
   ASSERT(uiIdxFromHead < m_uiDataSize);
   ASSERT(m_uiDataSize <= m_uiMaxQueueSize);
   ASSERT(m_iTailPointer < (int)m_uiMaxQueueSize);
   
   int iResPointer = m_iTailPointer-(int)m_uiDataSize+(int)uiIdxFromHead+1;
   
   if(iResPointer<0)
      iResPointer += (int)m_uiDataSize;
   else
      if(iResPointer >= (int)m_uiDataSize)
         iResPointer -= (int)m_uiDataSize;
   
   return(m_atObjects[iResPointer]);      
};

template<typename T>
T CQueueT::GetObjectFromTail(uint uiIdxFromTail) const
{
   ASSERT(m_uiDataSize > 0);
   ASSERT(uiIdxFromTail < m_uiDataSize);
   ASSERT(m_uiDataSize <= m_uiMaxQueueSize);
   ASSERT(m_iTailPointer < (int)m_uiMaxQueueSize);
   
   int iResPointer = m_iTailPointer-(int)uiIdxFromTail;
   
   if(iResPointer<0)
      iResPointer += (int)m_uiDataSize;
   
   ASSERT(iResPointer < ArraySize(m_atObjects));
   
   return(m_atObjects[iResPointer]);      
};


template<typename T>
bool CQueueT::RemoveFromHead(T & tObject)
{
   ASSERT(m_uiDataSize <= m_uiMaxQueueSize);
   ASSERT(m_iTailPointer < (int)m_uiMaxQueueSize);

   if(m_uiDataSize == 0)
      return(false);

   int iHeadPointer = m_iTailPointer-(int)m_uiDataSize+1;
   
   if(iHeadPointer<0)
      iHeadPointer += (int)m_uiDataSize;

   ASSERT(iHeadPointer < ArraySize(m_atObjects));

   tObject = m_atObjects[iHeadPointer]; 
   
   --m_uiDataSize;
   
   if(m_uiDataSize == 0)
      m_iTailPointer = INT_MAX;
   
   return(true);         
};

