#property library

#include <MyLib\DebugOrRelease\DebugSupport.mqh>
#include <MyLib\Arrays\StackT.mqh>

template<typename T>
CStackT::CStackT()
{
   SetMyObjectType(MOT_STACK_T);
   m_uiMaxStackSize = 0;
   m_uiDataSize = 0;
   m_uiPointer = UINT_MAX;
};

template<typename T>
bool CStackT::Init(uint uiMaxStackSize)
{
   ASSERT(uiMaxStackSize > 0);
   
   int iResSize = ArrayResize(m_atObjects,uiMaxStackSize);
   
   if(iResSize != uiMaxStackSize)
      return(false);
      
   m_uiMaxStackSize =  uiMaxStackSize;
   m_uiDataSize = 0;
   m_uiPointer = UINT_MAX;
   
   return(true);  
};

template<typename T>
T CStackT::GetObject(uint uiIdxFromBottom) const
{
   ASSERT(m_uiDataSize > 0);
   ASSERT(uiIdxFromBottom < m_uiDataSize);
   ASSERT(m_uiDataSize <= m_uiMaxStackSize);
   ASSERT(m_uiPointer < m_uiMaxStackSize);
   
   if(uiIdxFromBottom <= m_uiPointer)
      return(m_atObjects[m_uiPointer-uiIdxFromBottom]);
   
   return(m_atObjects[m_uiPointer+m_uiDataSize-uiIdxFromBottom]);      
};

template<typename T>
bool CStackT::AddToBottom(T & tObject)
{
   ASSERT(m_uiMaxStackSize > 0);
   ASSERT(ArraySize(m_atObjects) == m_uiMaxStackSize);

   if(m_uiDataSize == 0)
      {
      m_uiPointer = 0;
      m_atObjects[0] = tObject; 
      m_uiDataSize = 1;
      return(true);
      };

   ASSERT(m_uiDataSize <= m_uiMaxStackSize);
   ASSERT(m_uiPointer < m_uiMaxStackSize);

   bool bRes = false;

   ++m_uiPointer;
   
   if(m_uiPointer == m_uiMaxStackSize)
      m_uiPointer = 0;
      
   if(m_uiDataSize<m_uiMaxStackSize)
      {
      ++m_uiDataSize;
      bRes = true;
      };

   m_atObjects[m_uiPointer] = tObject; 

   return(bRes);
};


template<typename T>
bool CStackT::RemoveFromBottom(T & tObject)
{
   if(m_uiDataSize == 0)
      return(false);

   tObject = m_atObjects[m_uiPointer]; 
   
   --m_uiDataSize;
   
   if(m_uiPointer > 0)
      --m_uiPointer;
   else   
      m_uiPointer = m_uiMaxStackSize;
   
   return(true);         
};
