#property library

#include <MyLib\Arrays\XYPointsArray.mqh>
#include <MyLib\Common\IniCore.mq5>
#include <MyLib\DataStructures\StringMatrix.mq5>

static const string CXYPointsArray::COMMON_SECTION = "Common";
static const string CXYPointsArray::TOTAL_ENTRY = "Total";
static const string CXYPointsArray::POINT_SECTION_PREFIX = "Pt_";
static const string CXYPointsArray::X_ENTRY = "X";
static const string CXYPointsArray::Y_ENTRY = "Y";

CXYPointsArray::CXYPointsArray(uint uiAddBy):CStructArray(uiAddBy)
{
   SetMyObjectType(MOT_XYPOINTS_ARRAY);
};

void CXYPointsArray::PrintArray(int iDoublePrintPrec) const
{
   if(Total() == NULL)
      {
      Print("XYPoint Array is empty");
      return;
      };
      
   for(uint uiI=0; uiI<Total(); ++uiI)
      Print("X = " + DoubleToString(GetAt(uiI).m_dX,iDoublePrintPrec) + "; Y = " + DoubleToString(GetAt(uiI).m_dY,iDoublePrintPrec));
};

bool CXYPointsArray::Store(CINIFileI* pifINIFile,bool bDeleteContents) const
{
   ASSERT(pifINIFile!=NULL);
   
   if(bDeleteContents)
      pifINIFile.DeleteContents();
   
   RETURN_IF_FALSE(CINICore::CheckRes(pifINIFile.WriteLong(COMMON_SECTION,TOTAL_ENTRY,Total()))); 
   
   SXYPoint xpPoint;
   string strSectionName;
   
   for(uint uiI=0; uiI<Total(); ++uiI)
      {
      xpPoint = GetAt(uiI);
      
      strSectionName = POINT_SECTION_PREFIX + IntegerToString(uiI);
   
      RETURN_IF_FALSE(CINICore::CheckRes(pifINIFile.WriteDouble(strSectionName,X_ENTRY,xpPoint.m_dX,MAX_INI_DOUBLE_PRECISION))); 
      RETURN_IF_FALSE(CINICore::CheckRes(pifINIFile.WriteDouble(strSectionName,Y_ENTRY,xpPoint.m_dY,MAX_INI_DOUBLE_PRECISION))); 
      };   
   
   return(true);
  
};

bool CXYPointsArray::Load(CINIFileI* pifINIFile)
{
   ASSERT(pifINIFile!=NULL);
   
   DeleteContents();

   uint uiTotal = NULL;

   RETURN_IF_FALSE(pifINIFile.ReadUint(COMMON_SECTION,TOTAL_ENTRY,uiTotal));   

   SXYPoint xpPoint;
   string strSecionName;
   
   for(uint uiI=0; uiI<uiTotal; ++uiI)
      {
      strSecionName = POINT_SECTION_PREFIX + IntegerToString(uiI);
      
      RETURN_IF_FALSE(pifINIFile.ReadDouble(strSecionName,X_ENTRY,xpPoint.m_dX));   
      RETURN_IF_FALSE(pifINIFile.ReadDouble(strSecionName,Y_ENTRY,xpPoint.m_dY));   
      RETURN_IF_FALSE(AddElement(xpPoint));
      };

   return(true);
};

bool CXYPointsArray::ExportCSV(CStringMatrix* psmStrMatrix,bool bNeedHeader) const
{
   ASSERT_MYPOINTER(psmStrMatrix);
   
   psmStrMatrix.DeleteContents();  
   
   uint uiStrings = Total();

   if(bNeedHeader)
      ++uiStrings;

   if(uiStrings == NULL)
      return(true);
   

   RETURN_IF_FALSE(psmStrMatrix.Init(uiStrings,2));
   
   uint uiRowShift = 0;
   
   if(bNeedHeader)
      {
      ++uiRowShift;
      psmStrMatrix.SetElement(0,0,X_ENTRY);
      psmStrMatrix.SetElement(0,1,Y_ENTRY);
      };   

   SXYPoint xpPoint;
      
   for(uint uiI=0; uiI<Total(); ++uiI)
      {
      xpPoint = GetAt(uiI);
      
      psmStrMatrix.SetElement(uiI+uiRowShift,0,DoubleToString(xpPoint.m_dX,MAX_INI_DOUBLE_PRECISION));
      psmStrMatrix.SetElement(uiI+uiRowShift,1,DoubleToString(xpPoint.m_dY,MAX_INI_DOUBLE_PRECISION));
      };      

   return(true);         
};
