#include <MyLib\DebugOrRelease\DebugSupport.mqh>

/*
  :

double _ERF(double dX)                    //   (    ) 
double _ERFC(double dX)                   //   
double _NormalDistribution(double dX)     //  (x),   Excel =()
double _InvNormalDistribution(double y0)  //    probit(p),   (x),   Excel =()

*/

/* 
Error function
 
The integral is
 
                          x
                           -
                2         | |          2
  erf(x)  =  --------     |    exp( - t  ) dt.
             sqrt(pi)   | |
                         -
                          0
 
For 0 <= |x| < 1, erf(x) = x * P4(x**2)/Q5(x**2); otherwise
erf(x) = 1 - erfc(x).
 
 
ACCURACY:
 
                     Relative error:
arithmetic   domain     # trials      peak         rms
   IEEE      0,1         30000       3.7e-16     1.0e-16
 
*/

double _ERF(double dX)  //   (    ) 
{
   double dXSq = EMPTY_VALUE;
   double dS = 0;
   double dP = EMPTY_VALUE;
   double dQ = EMPTY_VALUE;

   if(dX > 0)
      dS = 1;
   else
      if(dX<0)
         dS = -1;   
   
   dX = MathAbs(dX);
   
   
   if(dX<0.5)
      {
      dXSq = dX*dX;
      dP = 0.007547728033418631287834;
      dP = 0.288805137207594084924010+dXSq*dP;
      dP = 14.3383842191748205576712+dXSq*dP;
      dP = 38.0140318123903008244444+dXSq*dP;
      dP = 3017.82788536507577809226+dXSq*dP;
      dP = 7404.07142710151470082064+dXSq*dP;
      dP = 80437.3630960840172832162+dXSq*dP;
      
      dQ = 0.0;
      dQ = 1.00000000000000000000000+dXSq*dQ;
      dQ = 38.0190713951939403753468+dXSq*dQ;
      dQ = 658.070155459240506326937+dXSq*dQ;
      dQ = 6379.60017324428279487120+dXSq*dQ;
      dQ = 34216.5257924628539769006+dXSq*dQ;
      dQ = 80437.3630960840172826266+dXSq*dQ;
      
      return(dS*1.1283791670955125738961589031*dX*dP/dQ);
      };
    
    
    if (dX>=10)
      return(dS);
    
    return(dS*(1.0-_ERFC(dX)));
};

/* 
 
(*************************************************************************
Complementary error function
 
 1 - erf(x) =
 
                          inf.
                            -
                 2         | |          2
  erfc(x)  =  --------     |    exp( - t  ) dt
              sqrt(pi)   | |
                          -
                           x
 
 
For small x, erfc(x) = 1 - erf(x); otherwise rational
approximations are computed.
 
 
ACCURACY:
 
                     Relative error:
arithmetic   domain     # trials      peak         rms
   IEEE      0,26.6417   30000       5.7e-14     1.5e-14
 
 
*************************************************************************)
*/

double _ERFC(double dX)  //   
{
   double dP = EMPTY_VALUE;
   double dQ = EMPTY_VALUE;

   if(dX<0)
      return(2.0 - _ERFC(-dX));
    
   if(dX<0.5)
      return(1.0 -_ERF(dX));
    
   if(dX>=10)
      return(0);

   dP = 0.0;
   dP = 0.5641877825507397413087057563+dX*dP;
   dP = 9.675807882987265400604202961+dX*dP;
   dP = 77.08161730368428609781633646+dX*dP;
   dP = 368.5196154710010637133875746+dX*dP;
   dP = 1143.262070703886173606073338+dX*dP;
   dP = 2320.439590251635247384768711+dX*dP;
   dP = 2898.0293292167655611275846+dX*dP;
   dP = 1826.3348842295112592168999+dX*dP;
   
   dQ = 1.0;
   dQ = 17.14980943627607849376131193+dX*dQ;
   dQ = 137.1255960500622202878443578+dX*dQ;
   dQ = 661.7361207107653469211984771+dX*dQ;
   dQ = 2094.384367789539593790281779+dX*dQ;
   dQ = 4429.612803883682726711528526+dX*dQ;
   dQ = 6089.5424232724435504633068+dX*dQ;
   dQ = 4958.82756472114071495438422+dX*dQ;
   dQ = 1826.3348842295112595576438+dX*dQ;
   
   return(MathExp(-(dX*dX))*dP/dQ);
};
 
/* 
(*************************************************************************
Normal distribution function
 
Returns the area under the Gaussian probability density
function, integrated from minus infinity to x:
 
                           x
                            -
                  1        | |          2
   ndtr(x)  = ---------    |    exp( - t /2 ) dt
              sqrt(2pi)  | |
                          -
                         -inf.
 
            =  ( 1 + erf(z) ) / 2
            =  erfc(z) / 2
 
where z = x/sqrt(2). Computation is via the functions
erf and erfc.
 
 
ACCURACY:
 
                     Relative error:
arithmetic   domain     # trials      peak         rms
   IEEE     -13,0        30000       3.4e-14     6.7e-15
 
 
*************************************************************************)
*/

double _NormalDistribution(double dX)  //  (x),   Excel =()
{
   return(0.5*(_ERF(dX/1.41421356237309504880)+1));
}; 

/* 
(*************************************************************************
Inverse of Normal distribution function
 
Returns the argument, x, for which the area under the
Gaussian probability density function (integrated from
minus infinity to x) is equal to y.
 
 
For small arguments 0 < y < exp(-2), the program computes
z = sqrt( -2.0 * log(y) );  then the approximation is
x = z - log(z)/z  - (1/z) P(1/z) / Q(1/z).
There are two rational functions P/Q, one for 0 < y < exp(-32)
and the other for y up to exp(-2).  For larger arguments,
w = y - 0.5, and  x/sqrt(2pi) = w + w**3 R(w**2)/S(w**2)).
 
ACCURACY:
 
                     Relative error:
arithmetic   domain        # trials      peak         rms
   IEEE     0.125, 1        20000       7.2e-16     1.3e-16
   IEEE     3e-308, 0.135   50000       4.6e-16     9.8e-17
 
 
*************************************************************************)

*/

double _InvNormalDistribution(double y0) //    probit(p),   (x),   Excel =(A2)
{
    double Expm2;
    double S2Pi;
    double x;
    double y;
    double z;
    double y2;
    double x0;
    double x1;
    int code;
    double P0;
    double Q0;
    double P1;
    double Q1;
    double P2;
    double Q2;

    Expm2 = 0.13533528323661269189;
    S2Pi = 2.50662827463100050242;
    
    if(y0<=0)
      return(-DBL_MAX);
        
    if(y0>=1)
      return(DBL_MAX);
    
    
    code = 1;
    y = y0;
    if(y>1.0-Expm2)
      {
      y = 1.0-y;
      code = 0;
      };
    
    if(y>Expm2)
      {
      y  = y-0.5;
      y2 = y*y;
      P0 = -59.9633501014107895267;
      P0 = 98.0010754185999661536+y2*P0;
      P0 = -56.6762857469070293439+y2*P0;
      P0 = 13.9312609387279679503+y2*P0;
      P0 = -1.23916583867381258016+y2*P0;
      Q0 = 1;
      Q0 = 1.95448858338141759834+y2*Q0;
      Q0 = 4.67627912898881538453+y2*Q0;
      Q0 = 86.3602421390890590575+y2*Q0;
      Q0 = -225.462687854119370527+y2*Q0;
      Q0 = 200.260212380060660359+y2*Q0;
      Q0 = -82.0372256168333339912+y2*Q0;
      Q0 = 15.9056225126211695515+y2*Q0;
      Q0 = -1.18331621121330003142+y2*Q0;
      x = y+y*y2*P0/Q0;
      x = x*S2Pi;
      return(x);
      };
      
    x = MathSqrt(-2.0*MathLog(y));
    x0 = x-MathLog(x)/x;
    z = 1.0/x;
    if(x<8.0)
      {
        P1 = 4.05544892305962419923;
        P1 = 31.5251094599893866154+z*P1;
        P1 = 57.1628192246421288162+z*P1;
        P1 = 44.0805073893200834700+z*P1;
        P1 = 14.6849561928858024014+z*P1;
        P1 = 2.18663306850790267539+z*P1;
        P1 = -1.40256079171354495875*0.1+z*P1;
        P1 = -3.50424626827848203418*0.01+z*P1;
        P1 = -8.57456785154685413611*0.0001+z*P1;
        Q1 = 1;
        Q1 = 15.7799883256466749731+z*Q1;
        Q1 = 45.3907635128879210584+z*Q1;
        Q1 = 41.3172038254672030440+z*Q1;
        Q1 = 15.0425385692907503408+z*Q1;
        Q1 = 2.50464946208309415979+z*Q1;
        Q1 = -1.42182922854787788574*0.1+z*Q1;
        Q1 = -3.80806407691578277194*0.01+z*Q1;
        Q1 = -9.33259480895457427372*0.0001+z*Q1;
        x1 = z*P1/Q1;
      }
    else
      {
        P2 = 3.23774891776946035970;
        P2 = 6.91522889068984211695+z*P2;
        P2 = 3.93881025292474443415+z*P2;
        P2 = 1.33303460815807542389+z*P2;
        P2 = 2.01485389549179081538*0.1+z*P2;
        P2 = 1.23716634817820021358*0.01+z*P2;
        P2 = 3.01581553508235416007*0.0001+z*P2;
        P2 = 2.65806974686737550832*0.000001+z*P2;
        P2 = 6.23974539184983293730*0.000000001+z*P2;
        Q2 = 1;
        Q2 = 6.02427039364742014255+z*Q2;
        Q2 = 3.67983563856160859403+z*Q2;
        Q2 = 1.37702099489081330271+z*Q2;
        Q2 = 2.16236993594496635890*0.1+z*Q2;
        Q2 = 1.34204006088543189037*0.01+z*Q2;
        Q2 = 3.28014464682127739104*0.0001+z*Q2;
        Q2 = 2.89247864745380683936*0.000001+z*Q2;
        Q2 = 6.79019408009981274425*0.000000001+z*Q2;
        x1 = z*P2/Q2;
      };
    x = x0-x1;
    if(code != 0)
      x = -x;
      
    return(x);
};

/*
(*************************************************************************
Inverse of the error function
 
 
*************************************************************************)
*/
double _InvERF(double dE)
{
   return(_InvNormalDistribution(0.5*(dE+1))/MathSqrt(2));
};
 
